/*******************************************************************************
 * Copyright 2009 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Seapine.SurroundSCM.API;

namespace CM_Toolbox
{
    public partial class frmMain : Form
    {
        string cmUser;
        string cmPassword;
        string ttServer;
        string scmServer;
        string scmPort;

        public SSCMContext context;
        public ArrayList ttDataInfo;
        //public DataTable ttTable;
        ttEngine ttp;
        CMConfig config;

        public frmMain(CMConfig pconfig)
        {
            InitializeComponent();
            config = pconfig;
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            //Set the logger
            Logger.SetPath(Application.StartupPath);
            try
            {                
               
                cmUser = config.getUserNameFromFile();
                cmPassword = config.getPasswordFromFile();
                ttServer = config.getTTURLFromFile();
                scmServer = config.getSSCMServerFromFile();
                scmPort = config.getSSCMPortFromFile();

                String curRepo;
                String curBranch;
                String curMainline;
                //Get the values from the env vars from the client
                curBranch = System.Environment.GetEnvironmentVariable("SSCM_BRANCH");
                curRepo = System.Environment.GetEnvironmentVariable("SSCM_REPOSITORY");
                curMainline = System.Environment.GetEnvironmentVariable("SSCM_MAINLINE");
                //Populate the textboxes to let the user know where we are
                txtCurBranch.Text = curBranch;
                ToolTip branchTip = new ToolTip();
                branchTip.SetToolTip(txtCurBranch, txtCurBranch.Text);
                txtRepo.Text = curRepo;
                ToolTip repoTip = new ToolTip();
                repoTip.SetToolTip(txtRepo, txtRepo.Text);

                //Initialize the scm engine
                scmEngine scm = new scmEngine(cmUser, cmPassword, scmServer, scmPort);
                //Get all attach to defect info for all files in current repo and branch
                ArrayList scmData = scm.CheckSCMHistory(curRepo, curBranch, curMainline);
                //pass that info to build the defect list
                BuildDefectTable(scmData);
                
            }
            catch (Exception fcc)
            {
                string error = "General Exception reported:\n" + fcc.Message;
                MessageBox.Show(error,"CM Toolbox");
                Logger.WriteLine(error);
                Application.Exit();

            }
            
            
        }
        /*
         * method takes the array with scm attach to defect data
         * and calls the tt engine to query against TestTrack to 
         * get up-to-date status, summary and number (in case of a renumber)
         * */
        private void BuildDefectTable(ArrayList scmData)
        {
            try
            {
                //initialize the tt engine
                ttp = new ttEngine(cmUser, cmPassword, ttServer);
                dgSCCFiles.DataSource = null;
                //Build the ttTable by passing the scm array
                ttp.BuildTestTrackTable(scmData);
                dgDefects.DataSource = ttp.ttTable;
                dgDefects.Columns[0].Visible = false;
                dgDefects.Columns[1].Width = 100;
                dgDefects.Columns[2].Width = 60;
                dgDefects.Columns[3].Width = 550;
                dgDefects.Columns[4].Width = 100;
                for (int i = 0; i < dgDefects.Rows.Count; i++)
                {
                    dgDefects.Rows[i].Height = 18;
                }
            }
            catch (Exception fcc)
            {
                string error = "There was an error building the data tables.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error,"CM Toolbox");
                Logger.WriteLine(error);
            }
        }
        private void PopulateFileList()
        {
            try
            {
                if (dgDefects.SelectedRows.Count > 0)
                {
                    DataTable fileTable = ttp.sccFileTable;

                    DataTable defectFileTable = new DataTable();
                    DataColumn[] sccColumns = new DataColumn[3];
                    sccColumns[0] = new DataColumn();
                    sccColumns[0].ColumnName = "File Name";
                    sccColumns[1] = new DataColumn();
                    sccColumns[1].ColumnName = "Revision";
                    sccColumns[2] = new DataColumn();
                    sccColumns[2].ColumnName = "Fixed Date";

                    for (int i = 0; i < sccColumns.Length; i++)
                    {
                        defectFileTable.Columns.Add(sccColumns[i]);
                    }
                    for (int i = 0; i < fileTable.Rows.Count; i++)
                    {
                        String dnum = fileTable.Rows[i][0].ToString();
                        if (dgDefects.SelectedRows[0].Cells[0].Value.Equals(dnum))
                        {
                            DataRow sccFileRow = defectFileTable.NewRow();
                            sccFileRow["File Name"] = fileTable.Rows[i][1];
                            sccFileRow["Revision"] = fileTable.Rows[i][2];
                            sccFileRow["Fixed Date"] = fileTable.Rows[i][3];
                            defectFileTable.Rows.Add(sccFileRow);
                        }
                    }
                    dgSCCFiles.DataSource = defectFileTable;
                    dgSCCFiles.Columns[0].Width = 500;
                    dgSCCFiles.Columns[1].Width = 70;
                    dgSCCFiles.Columns[2].Width = 120;
                    for (int i = 0; i < dgSCCFiles.Rows.Count; i++)
                    {
                        dgSCCFiles.Rows[i].Height = 18;
                    }
                }
            }
            catch (Exception fcc)
            {
                string error = "There was an error populating the file list for the defect.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }


        }

        private void dgDefects_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            PopulateFileList();
        }

       

        
        
            
        

        
       
       
    }
}