/*******************************************************************************
 * Copyright 2009 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Collections;
using System.Text;
using Seapine.SurroundSCM.API;
using System.Windows.Forms;

namespace CM_Toolbox
{
    class scmEngine
    {
        public SSCMContext context;
        public ArrayList ttDataInfo;
        string scmUser;
        string scmPassword;
        string scmServer;
        uint scmPort;

        public scmEngine(String pscmUser, String pscmPassword, String pscmServer, String pscmPort)
        {
            scmUser = pscmUser;
            scmPassword = pscmPassword;
            scmServer = pscmServer;
            scmPort = Convert.ToUInt16(pscmPort);
        }
        //'Runner' method, initializes the array, logs into SSCM, sets context and then calls the method 
        //that actually will build the array with data
        public ArrayList CheckSCMHistory(String curRepo, String curBranch, String curMainline)
        {
            ttDataInfo = new ArrayList();
            try
            {
                if (login())
                {
                    SetContext(curBranch, curMainline);
                    CheckChildRepos(curRepo);
                    ttDataInfo = SortArray(ttDataInfo);
                    SSCMAPI.Disconnect(context);

                }
            }
            catch (Exception fcc)
            {
                string error = "There was an error Checking Surround SCM history.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }
            return ttDataInfo;
        }
        private void SetContext(String branch, String Mainline)
        {
            context.SetBranch(branch);
            context.SetMainline(Mainline);
        }
        private bool login()
        {
            bool isLoggedIn = false;
            try
            {
                SSCMResult result = SSCMAPI.Connect(scmServer, scmPort, scmUser, scmPassword, out context);
                if (result.IsOk())
                {
                    isLoggedIn = true;
                }
                else
                {
                    MessageBox.Show("Unable to log in.\nError reported: \n" + SSCMAPI.GetLastError(result),"CM Toolbox");                   
                }
            }
            catch (Exception fcc)
            {
                string error = "There was a problem logging in.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);

            }
            return isLoggedIn;
                

        }
        /*
         * Method that traverses through all repositories 
         * within the current repository in the current branch
         * 
         * */
        private void CheckChildRepos(string curRepo)
        {
            try
            {
                SSCMRepo[] repos = new SSCMRepo[0];
                SSCMResult result = SSCMAPI.ListSubRepos(context, curRepo, out repos);
                //Call method to get attach to defect info for current repo
                CheckFilesInRepo(curRepo);
                if (result.IsOk())
                {
                    //Now for each child repo, we call this method
                    for (int i = 0; i < repos.Length; i++)
                    {
                        CheckChildRepos(repos[i].GetRepository());
                    }
                }
                else
                {
                    MessageBox.Show("Error getting child repositories from" + curRepo + ".\nError reported: \n" + SSCMAPI.GetLastError(result));
                }
            }
            catch (Exception fcc)
            {
                string error = "There was an error checking repositories.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }

        }
        /*
         * Method that checks file by file in the given repo
         * for attach to defect events.  It parses out the
         * record id and project name and adds it to the tt info array
         * 
         * */
        private void CheckFilesInRepo(string curRepo)
        {
            try
            {
                SSCMFile[] scmfiles = new SSCMFile[0];
                SSCMResult fileResult = SSCMAPI.ListFilesInRepo(context, curRepo, out scmfiles);
                if (fileResult.IsOk())
                {
                    if (scmfiles.Length > 0)
                    {
                        for (int i = 0; i < scmfiles.Length; i++)
                        {
                            FileHistoryItem[] historyItems = new FileHistoryItem[0];
                            SSCMResult historyResult = SSCMAPI.FileHistory(context, curRepo, scmfiles[i].GetName(), false, false, FileAction.AttachToDefect, out historyItems);
                            if (historyResult.IsOk())
                            {
                                for (int j = 0; j < historyItems.Length; j++)
                                {
                                    //This will hold the defect number and project
                                    string[] ttData = new string[2];
                                    //This will hold the defect number, project, fullpath to file, version and timestamp
                                    string[] fileData = new string[5];
                                    //get defect number and project for current attach to defect event
                                    ttData = ParseComments(historyItems[j].GetComment());
                                    //fill the string array
                                    fileData[0] = ttData[0];
                                    fileData[1] = ttData[1];
                                    fileData[2] = scmfiles[i].GetRepository() + "/" + scmfiles[i].GetName();
                                    fileData[3] = historyItems[j].GetVersion().ToString();
                                    fileData[4] = historyItems[j].GetTimestamp().ToString();
                                    //add this to the 'master' list of all files
                                    ttDataInfo.Add(fileData);

                                }
                            }
                        }
                    }
                }
            }
            catch (Exception fcc)
            {
                string error = "There was an error getting file history.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }
        }
        //method to parse the attach to defect event
        private string[] ParseComments(string comments)
        {
            string[] ttData = new string[2];
            try
            {


                //Get the recordID of defect
                comments = comments.Substring(comments.IndexOf(";") + 1);
                int pos = comments.IndexOf(";");
                String defRecID = comments.Substring(comments.IndexOf(":") + 2, (comments.IndexOf(";") - (comments.IndexOf(":") + 2)));
                //  ttData[0] = new string(;
                ttData[0] = defRecID;
                comments = comments.Substring(comments.IndexOf(";") + 1);
                pos = comments.IndexOf("Database Name:");
                comments = comments.Substring(pos);
                pos = comments.IndexOf(";");
                String ttProjectName = comments.Substring(comments.IndexOf(":") + 2, (comments.IndexOf(";") - (comments.IndexOf(":") + 2)));
                //   ttData[1] = new string();
                ttData[1] = ttProjectName;
            }
            catch (Exception fcc)
            {
                string error = "There was an error parsing the comments.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }
            return ttData;

        }
        //Sorts the array based on project name
        private ArrayList SortArray(ArrayList arrayToSort)
        {
            ArrayComparer comparer = new ArrayComparer();
            arrayToSort.Sort(comparer);
            return arrayToSort;
        }
        class ArrayComparer : IComparer
        {
            public int Compare(object x, object y)
            {
                //x and y are arrays of strings
                // sort on the second item in each array
                string[] a1 = (string[])x;
                string[] a2 = (string[])y;
                return (a1[1].CompareTo(a2[1]));
            }

        }
    }
}
