/*******************************************************************************
 * Copyright 2009 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Collections;
using System.Text;
using System.Data;
using System.Windows.Forms;
/*
 * ttEngine class
 * 
 * author: cremerf@seapine.com
 * 
 * This class handles all interactions with TestTrack SDK
 * 
 * Main purpose is to take the list of defects and their project found 
 * in the Surround SCM history and match it againts TT to get the most 
 * up-to-date number, summary and status
 * 
 * */
namespace CM_Toolbox
{
    class ttEngine
    {
        //Declaring global variables that various methods may use
        String ttUser;
        String ttPassword;
        String ttProjectName;
        String ttServerURL;

        ttsoapcgi ttSoap;

        //Flag we use to see if some methods fail
        //using string so we can then log the error or display it to the user
        public String errMsg = "";
        
        long ttCookie;
       
        //The two data tables that will be used by the GUI
        public DataTable ttTable;
        public DataTable sccFileTable;

        public ttEngine(String pUser, String pPassword, String pURL)
        {
            errMsg = "";
            try
            {
                ttUser = pUser;
                ttPassword = pPassword;
                ttServerURL = pURL;
                ttSoap = new ttsoapcgi();                
            }
            catch (Exception fcc)
            {
                errMsg = "";
                errMsg = "Error while initializing process.";
                errMsg += "Error Reported:";
                errMsg += fcc.Message;
            }            
        }
        public void BuildTestTrackTable(ArrayList scmInfo)
        {
            try
            {
                //Define array list to hold defect numbers for current project
                ArrayList dNums = new ArrayList();
                bool exists = false;
                //reset ttProjectName & rows containing defect data
                ttProjectName = "";
                CRecordRowSoap[] rows = null;
                //Build our data tables            
                BuildDataTables();
                //flag we will use to check if there was an error with project
                bool projError = false;
                //Now, for each file record, we match it against TestTrack    
                for (int i = 0; i < scmInfo.Count; i++)
                {
                    string[] scmData = new string[5];
                    scmData = (string[])scmInfo[i];
                    //To build the scc table we don't need TestTrack, so let's fill that out
                    /*
                     * The structure of scmData
                     *  [0] = record id of defect
                        [1] = project name
                        [2] = repository and file
                        [3] = version
                        [4] = Timestamp
                     * 
                     * The structure of the row we'll fill in
                     * 
                     * [0] = record id of defect
                     * [1] = repository and file
                     * [2] = version
                     * [3] = timestamp
                     * */
                    DataRow fileRow = sccFileTable.NewRow();
                    fileRow["RecordID"] = scmData[0]; ;
                    fileRow["File Name"] = scmData[2];
                    fileRow["Revision"] = scmData[3];
                    fileRow["Fixed Date"] = scmData[4];
                    sccFileTable.Rows.Add(fileRow);

                    //Now we build the TestTrack table  (ttTable)

                    //In case we need to query a different project we may need to log out of current project and log into the other one
                    if (scmData[1] != ttProjectName)
                    {
                        dNums = new ArrayList();
                        logout();
                        ttProjectName = scmData[1];
                        login();
                        if (errMsg != "")
                        {
                            projError = true;
                        }
                        else
                        {
                            projError = false;
                            //If we were able to login to new project, reset defect list
                            rows = GetDefectList();
                        }
                    }
                    if (projError == false)
                    {
                        for (int j = 0; j < rows.Length; j++)
                        {
                            CRecordData[] defectRow = rows[j].row;
                            string dNum = defectRow[0].value;
                            for (int k = 0; k < dNums.Count; k++)
                            {
                                if (dNum.Equals(dNums[k].ToString()))
                                {
                                    exists = true;
                                }

                            }
                            if (exists == false)
                            {
                                if (defectRow[0].value == scmData[0])
                                {
                                    DataRow ttRow = ttTable.NewRow();
                                    ttRow["RecordID"] = dNum;
                                    ttRow["Project Name"] = ttProjectName;
                                    ttRow["Number"] = Convert.ToInt32(defectRow[1].value);
                                    ttRow["Summary"] = defectRow[2].value;
                                    ttRow["Status"] = defectRow[3].value;
                                    ttTable.Rows.Add(ttRow);
                                    dNums.Add(dNum);
                                }
                            }
                            exists = false;
                        }

                    }
                    else
                    {
                        string dNum = scmData[0];
                        for (int k = 0; k < dNums.Count; k++)
                        {
                            if (dNum.Equals(dNums[k].ToString()))
                            {
                                exists = true;
                            }
                        }
                        if (exists == false)
                        {
                            DataRow ttRow = ttTable.NewRow();
                            ttRow["RecordID"] = scmData[0];
                            ttRow["Project Name"] = ttProjectName;
                            ttRow["Number"] = Convert.ToInt32(scmData[0]);
                            ttRow["Summary"] = "<unable to determine>";
                            ttRow["Status"] = "<unable to determine>";
                            ttTable.Rows.Add(ttRow);
                        }

                    }
                }
                logout();
            }
            catch (Exception fcc)
            {
                string error = "There was an error querying TestTrack.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }
                                
            
        }
        public bool login()
        {
            
            bool setProject = false;
            bool isLoggedIn = false;
            try
            {
                ttSoap.Url = ttServerURL;
                CProject ttProject = new CProject();
                CProject[] ttProjectList = ttSoap.getProjectList(ttUser, ttPassword);
                for (int i = 0; i < ttProjectList.Length; i++)
                {
                    if (ttProjectList[i].database.name == ttProjectName)
                    {
                        ttProject = ttProjectList[i];
                        setProject = true;
                    }
                }
                if (setProject)
                {
                    ttCookie = ttSoap.ProjectLogon(ttProject, ttUser, ttPassword);
                    if (ttCookie > 0)
                    {
                        isLoggedIn = true;
                        Logger.WriteLine("Logged into TestTrack.");
                    }
                }
                else
                {
                    errMsg = "";
                    errMsg = ttProjectName + " not found!";
                }

            }
            catch (Exception fcc)
            {
                string error = "There was an error logging in to TestTrack.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }
            return isLoggedIn;
        }
        public void logout()
        {
            if (ttCookie > 0)
            {
                try
                {
                    ttSoap.DatabaseLogoff(ttCookie);
                }
                catch (Exception fcc)
                {
                    string error = "There was an error logging out of TestTrack.\nError reported:\n" + fcc.Message;
                    MessageBox.Show(error, "CM Toolbox");
                    Logger.WriteLine(error);
                }
            }
        }
        public void BuildDataTables()
        {
            /*Create the two tables we need
             * 
             * 1. sccFileTable - holds file data
             * 2. ttTable - holds defect data
             * 
             * Both have a 'key' relationship in that both have the defect record id

            */
            try
            {
                ttTable = new DataTable("Defects");
                DataColumn[] ttColumns = new DataColumn[5];
                ttColumns[0] = new DataColumn();
                ttColumns[0].ColumnName = "RecordID";
                ttColumns[1] = new DataColumn();
                ttColumns[1].ColumnName = "Project Name";
                ttColumns[2] = new DataColumn();
                ttColumns[2].ColumnName = "Number";
                ttColumns[2].DataType = System.Type.GetType("System.Int32");
                ttColumns[3] = new DataColumn();
                ttColumns[3].ColumnName = "Summary";
                ttColumns[4] = new DataColumn();
                ttColumns[4].ColumnName = "Status";

                for (int i = 0; i < ttColumns.Length; i++)
                {
                    ttTable.Columns.Add(ttColumns[i]);
                }

                sccFileTable = new DataTable("SCCFiles");
                DataColumn[] sccColumns = new DataColumn[4];
                sccColumns[0] = new DataColumn();
                sccColumns[0].ColumnName = "RecordID";
                sccColumns[1] = new DataColumn();
                sccColumns[1].ColumnName = "File Name";
                sccColumns[2] = new DataColumn();
                sccColumns[2].ColumnName = "Revision";
                sccColumns[3] = new DataColumn();
                sccColumns[3].ColumnName = "Fixed Date";

                for (int i = 0; i < sccColumns.Length; i++)
                {
                    sccFileTable.Columns.Add(sccColumns[i]);
                }
            }
            catch (Exception fcc)
            {
                string error = "There was an error building the data tables.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }
        }
        private CRecordRowSoap[] GetDefectList()
        {
            CRecordRowSoap[] rows = null;
            try
            {
                CTableColumn[] listColumns = new CTableColumn[4];
                listColumns[0] = new CTableColumn();
                listColumns[0].name = "Record ID";
                listColumns[1] = new CTableColumn();
                listColumns[1].name = "Number";
                listColumns[2] = new CTableColumn();
                listColumns[2].name = "Summary";
                listColumns[3] = new CTableColumn();
                listColumns[3].name = "Status";

                CRecordListSoap defectList = ttSoap.getRecordListForTable(ttCookie, "Defect", "", listColumns);
                rows = defectList.records;

            }
            catch (Exception fcc)
            {
                string error = "There was an error getting the list of defects from TestTrack.\nError reported:\n" + fcc.Message;
                MessageBox.Show(error, "CM Toolbox");
                Logger.WriteLine(error);
            }
            return rows;
        }
               
    }
}
